@@ -9,7 +9,9 @@ from .transports import (
     UvxStdioTransport,
     NpxStdioTransport,
     FastMCPTransport,
+    StreamableHttpTransport,
 )
+from .auth import OAuth, BearerAuth
 
 __all__ = [
     "Client",
@@ -22,4 +24,7 @@ __all__ = [
     "UvxStdioTransport",
     "NpxStdioTransport",
     "FastMCPTransport",
+    "StreamableHttpTransport",
+    "OAuth",
+    "BearerAuth",
 ]
