@@ -173,74 +173,6 @@ class TestHelperFunctions:
             "file.py:server",
         ]
 
-    def test_parse_file_path_simple(self):
-        """Test parsing simple file path."""
-        with (
-            patch("pathlib.Path.exists") as mock_exists,
-            patch("pathlib.Path.is_file") as mock_is_file,
-            patch("pathlib.Path.expanduser") as mock_expanduser,
-            patch("pathlib.Path.resolve") as mock_resolve,
-        ):
-            mock_exists.return_value = True
-            mock_is_file.return_value = True
-            mock_expanduser.return_value = Path("file.py")
-            mock_resolve.return_value = Path("file.py")
-
-            path, obj = cli._parse_file_path("file.py")
-            assert path == Path("file.py")
-            assert obj is None
-
-    def test_parse_file_path_with_object(self):
-        """Test parsing file path with object."""
-        with (
-            patch("pathlib.Path.exists") as mock_exists,
-            patch("pathlib.Path.is_file") as mock_is_file,
-            patch("pathlib.Path.expanduser") as mock_expanduser,
-            patch("pathlib.Path.resolve") as mock_resolve,
-        ):
-            mock_exists.return_value = True
-            mock_is_file.return_value = True
-            mock_expanduser.return_value = Path("file.py")
-            mock_resolve.return_value = Path("file.py")
-
-            path, obj = cli._parse_file_path("file.py:server")
-            assert path == Path("file.py")
-            assert obj == "server"
-
-    def test_parse_file_path_windows(self):
-        """Test parsing Windows file path."""
-        with (
-            patch("pathlib.Path.exists") as mock_exists,
-            patch("pathlib.Path.is_file") as mock_is_file,
-            patch("pathlib.Path.expanduser") as mock_expanduser,
-            patch("pathlib.Path.resolve") as mock_resolve,
-        ):
-            mock_exists.return_value = True
-            mock_is_file.return_value = True
-            mock_expanduser.return_value = Path("C:/path/file.py")
-            mock_resolve.return_value = Path("C:/path/file.py")
-
-            path, obj = cli._parse_file_path("C:/path/file.py:server")
-            assert path == Path("C:/path/file.py")
-            assert obj == "server"
-
-    def test_parse_file_path_not_file(self, mock_exit, mock_logger):
-        """Test parsing path that is not a file."""
-        with (
-            patch("pathlib.Path.exists") as mock_exists,
-            patch("pathlib.Path.is_file") as mock_is_file,
-            patch("pathlib.Path.expanduser") as mock_expanduser,
-            patch("pathlib.Path.resolve") as mock_resolve,
-        ):
-            mock_exists.return_value = True
-            mock_is_file.return_value = False
-            mock_expanduser.return_value = Path("directory")
-            mock_resolve.return_value = Path("directory")
-
-            cli._parse_file_path("directory")
-            mock_logger.error.assert_called_once()
-            mock_exit.assert_called_once_with(1)
-
 
 class TestVersionCommand:
     """Tests for the version command."""
@@ -259,8 +191,8 @@ class TestDevCommand:
     def test_dev_command_success(self, temp_python_file, mock_logger):
         """Test successful dev command execution."""
         with (
-            patch("fastmcp.cli.cli._parse_file_path") as mock_parse,
-            patch("fastmcp.cli.cli._import_server") as mock_import,
+            patch("fastmcp.cli.run.parse_file_path") as mock_parse,
+            patch("fastmcp.cli.run.import_server") as mock_import,
             patch("fastmcp.cli.cli._get_npx_command") as mock_get_npx,
             patch("fastmcp.cli.cli._build_uv_command") as mock_build_uv,
             patch("subprocess.run") as mock_run,
@@ -277,16 +209,16 @@ class TestDevCommand:
             assert result.exit_code == 0
             mock_run.assert_called_once()
 
-            # Check dependencies were passed correctly
+            # Check dependencies were passed correctly with no_banner=True
             mock_build_uv.assert_called_once_with(
-                str(temp_python_file), None, ["extra_dep"]
+                str(temp_python_file), None, ["extra_dep"], no_banner=True
             )
 
     def test_dev_command_with_ui_port(self, temp_python_file):
         """Test dev command with UI port."""
         with (
-            patch("fastmcp.cli.cli._parse_file_path") as mock_parse,
-            patch("fastmcp.cli.cli._import_server") as mock_import,
+            patch("fastmcp.cli.run.parse_file_path") as mock_parse,
+            patch("fastmcp.cli.run.import_server") as mock_import,
             patch("fastmcp.cli.cli._get_npx_command") as mock_get_npx,
             patch("fastmcp.cli.cli._build_uv_command") as mock_build_uv,
             patch("subprocess.run") as mock_run,
@@ -310,8 +242,8 @@ class TestDevCommand:
     def test_dev_command_with_server_port(self, temp_python_file):
         """Test dev command with server port."""
         with (
-            patch("fastmcp.cli.cli._parse_file_path") as mock_parse,
-            patch("fastmcp.cli.cli._import_server") as mock_import,
+            patch("fastmcp.cli.run.parse_file_path") as mock_parse,
+            patch("fastmcp.cli.run.import_server") as mock_import,
             patch("fastmcp.cli.cli._get_npx_command") as mock_get_npx,
             patch("fastmcp.cli.cli._build_uv_command") as mock_build_uv,
             patch("subprocess.run") as mock_run,
@@ -335,8 +267,8 @@ class TestDevCommand:
     def test_dev_command_inspector_version(self, temp_python_file):
         """Test dev command with specific inspector version."""
         with (
-            patch("fastmcp.cli.cli._parse_file_path") as mock_parse,
-            patch("fastmcp.cli.cli._import_server") as mock_import,
+            patch("fastmcp.cli.run.parse_file_path") as mock_parse,
+            patch("fastmcp.cli.run.import_server") as mock_import,
             patch("fastmcp.cli.cli._get_npx_command") as mock_get_npx,
             patch("fastmcp.cli.cli._build_uv_command") as mock_build_uv,
             patch("subprocess.run") as mock_run,
@@ -360,11 +292,12 @@ class TestDevCommand:
 class TestRunCommand:
     """Tests for the run command."""
 
-    def test_run_command_success(self, temp_python_file, mock_logger):
+    def test_run_command_success(self, temp_python_file):
         """Test successful run command execution."""
         with (
-            patch("fastmcp.cli.cli._parse_file_path") as mock_parse,
-            patch("fastmcp.cli.cli._import_server") as mock_import,
+            patch("fastmcp.cli.run.parse_file_path") as mock_parse,
+            patch("fastmcp.cli.run.import_server") as mock_import,
+            patch("fastmcp.cli.run.logger") as mock_logger,
         ):
             mock_parse.return_value = (temp_python_file, None)
             mock_server = MagicMock()
@@ -374,15 +307,15 @@ class TestRunCommand:
             result = runner.invoke(cli.app, ["run", str(temp_python_file)])
             assert result.exit_code == 0
             mock_server.run.assert_called_once_with()
-            mock_logger.info.assert_called_with(
+            mock_logger.debug.assert_called_with(
                 f'Found server "test_server" in {temp_python_file}'
             )
 
     def test_run_command_with_transport(self, temp_python_file):
         """Test run command with transport option."""
         with (
-            patch("fastmcp.cli.cli._parse_file_path") as mock_parse,
-            patch("fastmcp.cli.cli._import_server") as mock_import,
+            patch("fastmcp.cli.run.parse_file_path") as mock_parse,
+            patch("fastmcp.cli.run.import_server") as mock_import,
         ):
             mock_parse.return_value = (temp_python_file, None)
             mock_server = MagicMock()
@@ -395,11 +328,46 @@ class TestRunCommand:
             assert result.exit_code == 0
             mock_server.run.assert_called_once_with(transport="sse")
 
+    def test_run_command_with_http_transports(self, temp_python_file):
+        """Test run command with both http and streamable-http transport options."""
+        # Test "http" transport
+        with (
+            patch("fastmcp.cli.run.parse_file_path") as mock_parse,
+            patch("fastmcp.cli.run.import_server") as mock_import,
+        ):
+            mock_parse.return_value = (temp_python_file, None)
+            mock_server = MagicMock()
+            mock_server.name = "test_server"
+            mock_import.return_value = mock_server
+
+            result = runner.invoke(
+                cli.app, ["run", str(temp_python_file), "--transport", "http"]
+            )
+            assert result.exit_code == 0
+            mock_server.run.assert_called_once_with(transport="http")
+
+        # Test "streamable-http" transport (alias for http)
+        with (
+            patch("fastmcp.cli.run.parse_file_path") as mock_parse,
+            patch("fastmcp.cli.run.import_server") as mock_import,
+        ):
+            mock_parse.return_value = (temp_python_file, None)
+            mock_server = MagicMock()
+            mock_server.name = "test_server"
+            mock_import.return_value = mock_server
+
+            result = runner.invoke(
+                cli.app,
+                ["run", str(temp_python_file), "--transport", "streamable-http"],
+            )
+            assert result.exit_code == 0
+            mock_server.run.assert_called_once_with(transport="streamable-http")
+
     def test_run_command_with_host(self, temp_python_file):
         """Test run command with host option."""
         with (
-            patch("fastmcp.cli.cli._parse_file_path") as mock_parse,
-            patch("fastmcp.cli.cli._import_server") as mock_import,
+            patch("fastmcp.cli.run.parse_file_path") as mock_parse,
+            patch("fastmcp.cli.run.import_server") as mock_import,
         ):
             mock_parse.return_value = (temp_python_file, None)
             mock_server = MagicMock()
@@ -415,8 +383,8 @@ class TestRunCommand:
     def test_run_command_with_port(self, temp_python_file):
         """Test run command with port option."""
         with (
-            patch("fastmcp.cli.cli._parse_file_path") as mock_parse,
-            patch("fastmcp.cli.cli._import_server") as mock_import,
+            patch("fastmcp.cli.run.parse_file_path") as mock_parse,
+            patch("fastmcp.cli.run.import_server") as mock_import,
         ):
             mock_parse.return_value = (temp_python_file, None)
             mock_server = MagicMock()
@@ -432,8 +400,8 @@ class TestRunCommand:
     def test_run_command_with_log_level(self, temp_python_file):
         """Test run command with log level option."""
         with (
-            patch("fastmcp.cli.cli._parse_file_path") as mock_parse,
-            patch("fastmcp.cli.cli._import_server") as mock_import,
+            patch("fastmcp.cli.run.parse_file_path") as mock_parse,
+            patch("fastmcp.cli.run.import_server") as mock_import,
         ):
             mock_parse.return_value = (temp_python_file, None)
             mock_server = MagicMock()
@@ -449,8 +417,8 @@ class TestRunCommand:
     def test_run_command_with_multiple_options(self, temp_python_file):
         """Test run command with multiple options."""
         with (
-            patch("fastmcp.cli.cli._parse_file_path") as mock_parse,
-            patch("fastmcp.cli.cli._import_server") as mock_import,
+            patch("fastmcp.cli.run.parse_file_path") as mock_parse,
+            patch("fastmcp.cli.run.import_server") as mock_import,
         ):
             mock_parse.return_value = (temp_python_file, None)
             mock_server = MagicMock()
@@ -476,3 +444,29 @@ class TestRunCommand:
             mock_server.run.assert_called_once_with(
                 transport="sse", host="0.0.0.0", port=8080, log_level="DEBUG"
             )
+
+    def test_run_command_with_server_args(self, temp_python_file):
+        """Test run command with server arguments using -- pattern."""
+        with (
+            patch("fastmcp.cli.run.run_command") as mock_run_command,
+        ):
+            result = runner.invoke(
+                cli.app,
+                [
+                    "run",
+                    str(temp_python_file),
+                    "--",
+                    "--config",
+                    "config.json",
+                ],
+            )
+            assert result.exit_code == 0
+            mock_run_command.assert_called_once_with(
+                server_spec=str(temp_python_file),
+                transport=None,
+                host=None,
+                port=None,
+                log_level=None,
+                server_args=["--config", "config.json"],
+                show_banner=True,
+            )
